#! /usr/bin/perl
#**********************************************************************
#* (C) Copyright IBM Corporation 2004, 2004.  All rights reserved.    *
#>                                                                    *
#* Tivoli Storage Manager                                             *
#*                                                                    *
#* This script defines all registered nodes in domain_name that are   *
#* not currently defined as members of a collocation group to the     *
#* group group_name.  The collocation group group_name is created if  *
#* it does not already exist.                                         *
#*                                                                    *
#* Usage: addnodestogroup -ID=userid -PAssword=password               *
#*                  domain_name group_name                            *
#*                                                                    *
#*        where -ID=userid specifies the administrator's user ID      *
#*              -PAssword=password specifies the administrator's      *
#*                 password                                           *
#*              domain_name specifies the TSM policy domain name      *
#*              group_name specifies the collocation group name       *
#*                                                                    *
#* Dependencies: UNIX or Windows                                      *
#*               Perl interpreter installed                           *
#*               TSM administrative client installed and configured   *
#*               TSM 5.3.0 server or higher                           *
#>                                                                    *
#* Note: This sample script is shipped "AS-IS" with all TSM server    *
#* platforms and must remain formatted with lines less than 80        *
#* characters each.                                                   *
#*                                                                    *
#**********************************************************************
use strict;
if ( @ARGV == 0 ) { usage() }

my $Id = "";
my $Pa = "";
while ( substr( $ARGV[0], 0, 1 ) eq "-" ) {
   my $A = uc( $ARGV[0] );
   if ( chkarg( $A, "-ID", 3 ) )       { $Id = $ARGV[0]; shift; next; }
   if ( chkarg( $A, "-PASSWORD", 3 ) ) { $Pa = $ARGV[0]; shift; next; }
   if ( $ARGV[0] eq "-h" ) { usage() }
   print "Syntax error: Argument $ARGV[0] not understood.\n"; shift;
}

if ( $Id eq '' ) {
   print "Syntax error: TSM administrator's user ID must be supplied.\n";
   usage();
   exit;
}
if ( $Pa eq '' ) {
   print "Syntax error: TSM administrator's password must be supplied.\n";
   usage();
   exit;
}
if ( @ARGV < 2 ) {
   print "Syntax error: Missing required arguments.\n";
   usage();
   exit;
}
if ( @ARGV > 2 ) {
   print "Syntax error: Extraneous arguments supplied.\n";
   usage();
   exit;
}

my $Domain = uc( $ARGV[0] );
my $Group  = uc( $ARGV[1] );
my $Macro  = "_addndes.mac";
my @Nodes  = ();

getnodes();
if ( @Nodes == 0 ) {
   print "There are no registered nodes in policy domain '$Domain' ";
   print "that need to be added to a collocation group.\n";
   exit;
}

addgroup();
addnodes();
exit;

sub usage
#**********************************************************************
#* Display program usage information by reading program code header   *
#*                                                                    *
#**********************************************************************
{
   my $Skip = 1;
   open USAGE, "<$0";
   while ( <USAGE> ) {
      if ( substr( $_, 0, 2 ) eq "#>" ) {
         if ( $Skip == 1 ) { $Skip = 0 }
         else              { last }
      }
      if ( $Skip == 0 ) { chop; chop; print substr( $_, 3 )."\n" }
   }
   close USAGE;
   exit 4;
}

sub chkarg
#**********************************************************************
#* Check supplied userid and password arguments                       *
#*                                                                    *
#**********************************************************************
{
   my ( $Arg, $Flag, $Length ) = @_;
   my @Arg = split "=", $Arg;
   if ( length( $Arg[0] ) > $Length ) { $Length = length( $Arg[0] ) }
   if ( $Arg[0] eq substr( $Flag, 0, $Length ) and $Arg[1] ne '' ) {
      return 1;
   }
   else {
      return 0;
   }
}

sub getnodes
#**********************************************************************
#* Get the list of nodes not currently defined as members of a        *
#* collocation group.                                                 *
#*                                                                    *
#**********************************************************************
{
   my ( $I, @Lines, $Opts );

   open  MACRO, ">$Macro" or die;
   print MACRO  "select '-', node_name from nodes where";
   print MACRO  " collocgroup_name is null and domain_name='$Domain'\n";
   close MACRO;

   $Opts = "$Id $Pa -noconfirm -commadelimited";
   @Lines = split "\n", `dsmadmc $Opts macro $Macro`;

   for ( $I = 0; $I < @Lines; $I++ ) {
      if ( substr( $Lines[$I], 0, 2 ) eq "-," ) {
         push @Nodes, substr( $Lines[$I], 2 );
      }
   }

   unlink "$Macro";
}

sub addgroup
#**********************************************************************
#* Try to add the collocation group.                                  *
#*                                                                    *
#**********************************************************************
{
   system "dsmadmc $Id $Pa -noconfirm define collocgroup $Group";
}

sub addnodes
#**********************************************************************
#* Try to define the nodes to be collocation group members.           *
#*                                                                    *
#**********************************************************************
{
   my $I;

   open  MACRO, ">$Macro" or die;
   for ( $I = 0; $I < @Nodes; $I++ ) {
      print MACRO  "define collocmember $Group $Nodes[$I]\n";
   }
   close MACRO;

   system "dsmadmc $Id $Pa -noconfirm -itemcommit macro $Macro";
   unlink "$Macro";
}
